@@ -103,9 +103,13 @@ get_ruleset (FT_Face face)
 
       ruleset = pango_ot_ruleset_new (info);
 
+      /* according to the Arabic OpenType spec, available here:
+       * http://www.microsoft.com/typography/otfntdev/arabicot/features.htm
+       */
       if (pango_ot_info_find_script (info, PANGO_OT_TABLE_GSUB,
 				     arab_tag, &script_index))
 	{
+	  /* Language based forms: */
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('c','c','m','p'), 0xFFFF);
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('i','s','o','l'), isolated);
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('f','i','n','a'), final);
@@ -113,13 +117,19 @@ get_ruleset (FT_Face face)
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('i','n','i','t'), initial);
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('r','l','i','g'), 0xFFFF);
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('c','a','l','t'), 0xFFFF);
+
+	  /* Typographical forms: */
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('l','i','g','a'), 0xFFFF);
+	  /* this one should be turned-on/off-able.  lets turn off for now. */
+	  /* maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('d','l','i','g'), 0xFFFF); */
+	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('c','s','w','h'), 0xFFFF);
 	  maybe_add_gsub_feature (ruleset, info, script_index, FT_MAKE_TAG ('m','s','e','t'), 0xFFFF);
 	}
 
       if (pango_ot_info_find_script (info, PANGO_OT_TABLE_GPOS,
 				     arab_tag, &script_index))
 	{
+	  /* Positioning features: */
 	  maybe_add_gpos_feature (ruleset, info, script_index, FT_MAKE_TAG ('c','u','r','s'), 0xFFFF);
 	  maybe_add_gpos_feature (ruleset, info, script_index, FT_MAKE_TAG ('k','e','r','n'), 0xFFFF);
 	  maybe_add_gpos_feature (ruleset, info, script_index, FT_MAKE_TAG ('m','a','r','k'), 0xFFFF);
@@ -169,8 +179,7 @@ fallback_shape (PangoEngineShape *engine,
 		PangoGlyphString *glyphs)
 {
   PangoFcFont *fc_font = PANGO_FC_FONT (font);
-  glong n_chars;
-  gunichar *wcs = g_utf8_to_ucs4_fast (text, length, &n_chars);
+  glong n_chars = g_utf8_strlen (text, length);
   const char *p;
   int i;
   
@@ -197,7 +206,7 @@ fallback_shape (PangoEngineShape *engine,
 	    input = buf;
 	  }
 
-      if (wc >= 0x200B && wc <= 0x200F)	/* Zero-width characters */
+      if (pango_is_zero_width (wc))
 	{
 	  set_glyph (font, glyphs, i, p - text, 0);
 	}
@@ -236,8 +245,6 @@ fallback_shape (PangoEngineShape *engine,
       /* Swap all glyphs */
       swap_range (glyphs, 0, glyphs->num_glyphs);
     }
-
-  g_free (wcs);
 }
 
 static void 
@@ -257,6 +264,7 @@ arabic_engine_shape (PangoEngineShape *engine,
   PangoOTRuleset *ruleset;
   PangoOTBuffer *buffer;
   PangoFcFont *fc_font;
+  int cluster = 0;
 
   g_return_if_fail (font != NULL);
   g_return_if_fail (text != NULL);
@@ -277,11 +285,14 @@ arabic_engine_shape (PangoEngineShape *engine,
 
   buffer = pango_ot_buffer_new (fc_font);
   pango_ot_buffer_set_rtl (buffer, analysis->level % 2 != 0);
+  pango_ot_buffer_set_zero_width_marks (buffer, TRUE);
     
   wcs = g_utf8_to_ucs4_fast (text, length, &n_chars);
   properties = g_new0 (gulong, n_chars);
       
   Arabic_Assign_Properties (wcs, properties, n_chars);
+
+  g_free (wcs);
   
   p = text;
   for (i=0; i < n_chars; i++)
@@ -291,7 +302,6 @@ arabic_engine_shape (PangoEngineShape *engine,
       PangoGlyph index;
       char buf[6];
       const char *input;
-      int cluster = 0;
 
       wc = g_utf8_get_char (p);
 
@@ -314,9 +324,10 @@ arabic_engine_shape (PangoEngineShape *engine,
 	  /* Hack - Microsoft fonts are strange and don't contain the
 	   * correct rules to shape ARABIC LETTER FARSI YEH in
 	   * medial/initial position. It looks identical to ARABIC LETTER
-	   * YEH in these positions, so we substitute
+	   * YEH in these positions, so we substitute if the font contains
+	   * ARABIC LETTER YEH
 	   */
-	  if (wc == 0x6cc && ruleset &&
+	  if (wc == 0x6cc && ruleset && pango_fc_font_get_glyph (fc_font, 0x64a) &&
 	      ((properties[i] & (initial | medial)) != (initial | medial)))
 	    wc = 0x64a;
 	  
@@ -344,7 +355,6 @@ arabic_engine_shape (PangoEngineShape *engine,
   pango_ot_ruleset_position (ruleset, buffer);
   pango_ot_buffer_output (buffer, glyphs);
   
-  g_free (wcs);
   g_free (properties);
   pango_ot_buffer_destroy (buffer);
 
