@@ -45,8 +45,8 @@ struct _PangoIndicInfo
 {
   PangoOTTag               scriptTag; 
   const IndicOTClassTable *classTable;
-  gchar                   *gsubQuarkName;
-  gchar                   *gposQuarkName;
+  const gchar             *gsubQuarkName;
+  const gchar             *gposQuarkName;
 };
 
 #define ENGINE_SUFFIX "ScriptEngineFc"
@@ -234,7 +234,7 @@ get_gpos_ruleset (FT_Face face, const PangoIndicInfo *indic_info)
 
       ruleset = pango_ot_ruleset_new (info);
 
-      if (pango_ot_info_find_script (info, PANGO_OT_TABLE_GPOS,
+      if (1 && pango_ot_info_find_script (info, PANGO_OT_TABLE_GPOS,
 				     indic_info->scriptTag, &script_index))
 	{
 	  maybe_add_GPOS_feature (ruleset, info, script_index, FT_MAKE_TAG ('b','l','w','m'), blwm);
@@ -250,19 +250,17 @@ get_gpos_ruleset (FT_Face face, const PangoIndicInfo *indic_info)
 }
 
 static void
-set_glyphs (PangoFont *font,
+set_glyphs (PangoFont      *font,
+	    FT_Face         face,
 	    const gunichar *wcs,
-	    gulong *tags,
-	    glong n_glyphs,
-	    PangoOTBuffer *buffer,
-	    gboolean process_zwj)
+	    gulong         *tags,
+	    glong           n_glyphs,
+	    PangoOTBuffer  *buffer,
+	    gboolean        process_zwj)
 {
   gint i;
-  PangoFcFont *fc_font;
 
-  g_assert (font);
-
-  fc_font = PANGO_FC_FONT (font);
+  g_assert (face);
 
   for (i = 0; i < n_glyphs; i += 1)
     {
@@ -272,14 +270,9 @@ set_glyphs (PangoFont *font,
 	  (!process_zwj || wcs[i] != 0x200D))
 	glyph = 0;
       else
-        {
-	  glyph = pango_fc_font_get_glyph (fc_font, wcs[i]);
-
-	  if (!glyph)
-	    glyph = pango_fc_font_get_unknown_glyph (fc_font, wcs[i]);
-
-          pango_ot_buffer_add_glyph (buffer, glyph, tags[i], i);
-	}
+	glyph = FT_Get_Char_Index (face, wcs[i]);
+      
+      pango_ot_buffer_add_glyph (buffer, glyph, tags[i], i);
     }
 }
 
@@ -300,7 +293,7 @@ expand_text(const gchar *text, glong length, glong **offsets, glong *n_chars)
   p   = text;
   wco = wcs;
   oo  = *offsets;
-  for (i = 0; i < *n_chars; i++)
+  for (i = 0; i < *n_chars; i += 1)
     {
       *wco++ = g_utf8_get_char (p);
       *oo++  = p - text;
@@ -362,7 +355,7 @@ indic_engine_shape (PangoEngineShape *engine,
   pango_glyph_string_set_size (glyphs, n_glyphs);
   buffer = pango_ot_buffer_new (fc_font);
 
-  set_glyphs(font, wc_out, tags, n_glyphs, buffer,
+  set_glyphs(font, face, wc_out, tags, n_glyphs, buffer,
 	     (indic_info->classTable->scriptFlags & SF_PROCESS_ZWJ) != 0);
 
   /* do gsub processing */
@@ -429,7 +422,7 @@ PANGO_MODULE_ENTRY(list) (PangoEngineInfo **engines,
 PangoEngine *
 PANGO_MODULE_ENTRY(create) (const char *id)
 {
-  gint i;
+  guint i;
 
   for (i = 0; i < G_N_ELEMENTS(script_engines); i += 1)
     {
