@@ -1161,6 +1161,8 @@ itemize_state_finish (ItemizeState *state)
  * is specified explicitely rather than gotten from the #PangoContext.
  *
  * Return value: a GList of PangoItem structures.
+ *
+ * Since: 1.4
  */
 GList *
 pango_itemize_with_base_dir (PangoContext      *context, 
@@ -1334,8 +1336,11 @@ update_metrics_from_items (PangoFontMetrics *metrics,
   
   g_hash_table_destroy (fonts_seen);
   
-  metrics->approximate_char_width /= count;
-  metrics->approximate_digit_width /= count;
+  if (count)
+    {
+      metrics->approximate_char_width /= count;
+      metrics->approximate_digit_width /= count;
+    }
 }
 
 /**
