@@ -208,7 +208,7 @@ pango_coverage_set (PangoCoverage     *coverage,
   
   g_return_if_fail (coverage != NULL);
   g_return_if_fail (index >= 0);
-  g_return_if_fail (level >= 0 || level <= 3);
+  g_return_if_fail (level >= 0 && level <= 3);
 
   block_index = index / 256;
 
@@ -217,7 +217,7 @@ pango_coverage_set (PangoCoverage     *coverage,
       int old_n_blocks = coverage->n_blocks;
 
       coverage->n_blocks =
-	N_BLOCKS_INCREMENT * ((block_index + N_BLOCKS_INCREMENT - 1) / N_BLOCKS_INCREMENT);
+	N_BLOCKS_INCREMENT * ((block_index + N_BLOCKS_INCREMENT) / N_BLOCKS_INCREMENT);
       
       coverage->blocks = g_renew (PangoBlockInfo, coverage->blocks, coverage->n_blocks);
       memset (coverage->blocks + old_n_blocks, 0,
@@ -439,7 +439,7 @@ pango_coverage_get_uint32 (guchar **ptr)
  * Convert data generated from pango_converage_to_bytes() back
  * to a #PangoCoverage
  * 
- * Return value: a newly allocated #PangoCoverage, or NULL if
+ * Return value: a newly allocated #PangoCoverage, or %NULL if
  *               the data was invalid.
  **/
 PangoCoverage *
