@@ -43,10 +43,10 @@ PANGO_DEFINE_TYPE_ABSTRACT (PangoFontset, pango_fontset,
 /**
  * pango_fontset_get_font:
  * @fontset: a #PangoFontset
- * @wc: a unicode character
+ * @wc: a Unicode character
  *
  * Returns the font in the fontset that contains the best glyph for the
- * unicode character wc.
+ * Unicode character @wc.
  *
  * Returns: a #PangoFont. The caller must call g_object_unref when finished
  *          with the font.
@@ -182,8 +182,11 @@ pango_fontset_real_get_metrics (PangoFontset  *fontset)
 
   g_hash_table_destroy (fonts_seen);
   
-  metrics->approximate_char_width /= count;
-  metrics->approximate_digit_width /= count;
+  if (count)
+    {
+      metrics->approximate_char_width /= count;
+      metrics->approximate_digit_width /= count;
+    }
 
   return metrics;
 }
@@ -274,7 +277,7 @@ pango_fontset_simple_finalize (GObject *object)
 {
   PangoFontsetSimple *fontset = PANGO_FONTSET_SIMPLE (object);
   PangoCoverage *coverage;
-  int i;
+  unsigned int i;
 
   for (i = 0; i < fontset->fonts->len; i++)
     g_object_unref (g_ptr_array_index(fontset->fonts, i));
@@ -352,7 +355,7 @@ pango_fontset_simple_get_font (PangoFontset  *fontset,
   PangoFont *font;
   PangoCoverage *coverage;
   int result = -1;
-  int i;
+  unsigned int i;
   
   for (i = 0; i < simple->fonts->len; i++)
     {
@@ -387,7 +390,7 @@ pango_fontset_simple_foreach (PangoFontset           *fontset,
 			      gpointer                data)
 {
   PangoFontsetSimple *simple = PANGO_FONTSET_SIMPLE (fontset);
-  int i;
+  unsigned int i;
   
   for (i = 0; i < simple->fonts->len; i++)
     {
