@@ -368,7 +368,9 @@ pango_ft2_font_real_get_unknown_glyph (PangoFcFont *font,
 }
 
 static gboolean
-pango_ft2_free_glyph_info_callback (gpointer key, gpointer value, gpointer data)
+pango_ft2_free_glyph_info_callback (gpointer key,
+				    gpointer value,
+				    gpointer data)
 {
   PangoFT2Font *font = PANGO_FT2_FONT (data);
   PangoFT2GlyphInfo *info = value;
@@ -466,7 +468,10 @@ _pango_ft2_ft_strerror (FT_Error error)
     return found->msg;
   else
     {
-      static char default_msg[100];
+      static char *default_msg = NULL;
+
+      if (!default_msg)
+        default_msg = g_malloc (60);
 
       g_sprintf (default_msg, "Unknown FreeType2 error %#x", error);
       return default_msg;
